@@ -1,5 +1,4 @@
 #!/bin/bash
-# $Id$
 # Copyright 2006 Michael Mohr with modifications by Roger Dingledine
 # See LICENSE for licensing information.
 
@@ -186,7 +185,7 @@ if [ ! -z $STRIP ]
 then
 ${HOST_TRIPLET}strip \
 	src/or/tor \
-	src/or/test \
+	src/test/test \
 	src/tools/tor-resolve
 fi
 
