@@ -1,6 +1,9 @@
-/* Copyright (c) 2007-2012, The Tor Project, Inc. */
+/* Copyright (c) 2007-2013, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
+#ifndef TOR_EVENTDNS_TOR_H
+#define TOR_EVENTDNS_TOR_H
+
 #include "orconfig.h"
 #define DNS_USE_OPENSSL_FOR_ID
 #ifndef HAVE_UINT
@@ -18,3 +21,5 @@ typedef unsigned char u_char;
 #include "util.h"
 #include "compat.h"
 
+#endif
+
